import { useState, useImperativeHandle, forwardRef } from 'react'

import { FormProvider, useForm } from 'react-hook-form'

import { Alert } from '@tribeplatform/react-ui-kit/Alert'

import { FormRef, FormProps } from './typings'

export const Form = forwardRef<FormRef, FormProps>(
  ({ defaultValues = {}, children, onSubmit, alertClasses, ...rest }, ref) => {
    const methods = useForm<Record<string, unknown>, Record<string, unknown>>({
      defaultValues,
    })
    const [generalErrors, setGeneralErrors] = useState([])
    const {
      setError,
      handleSubmit,
      formState: { isSubmitSuccessful },
    } = methods

    async function submitForm(data) {
      try {
        setGeneralErrors([])
        await onSubmit(data, methods)
      } catch (err) {
        const errors = err?.response?.errors
        const errMsgs = []
        errors?.forEach(e => {
          if (e?.field) {
            setError(e?.field as any, {
              type: 'validate',
              message: e?.message,
            })
          } else if (e?.message) {
            errMsgs.push(e)
          }
        })

        if (errMsgs.length) {
          setGeneralErrors(errMsgs)
        }
      }
    }

    useImperativeHandle(ref, () => ({
      methods,
    }))

    if (typeof children === 'function') {
      children = children({ ...methods, submitForm })
    }

    return (
      <FormProvider {...methods}>
        <form
          onSubmit={handleSubmit(submitForm)}
          data-submit-successful={isSubmitSuccessful || undefined}
          {...rest}
        >
          {generalErrors?.length ? (
            <Alert status="error" className={alertClasses || 'mb-3'}>
              {generalErrors.map(err => {
                if (!err?.message) return null
                return <div key={err?.message}>{err?.message}</div>
              })}
            </Alert>
          ) : null}

          {children}
        </form>
      </FormProvider>
    )
  },
)
