import { lazy, Suspense, useCallback, useState } from 'react'

import clsx from 'clsx'
import Parser, { domToReact, HTMLReactParserOptions } from 'html-react-parser'
import { Helmet } from 'react-helmet'
import { Link as RouterLink } from 'react-router-dom'
// eslint-disable-next-line no-restricted-imports
import atomOneDark from 'react-syntax-highlighter/dist/cjs/styles/hljs/atom-one-dark-reasonable'

import { Embed, Member } from '@tribeplatform/gql-client/types'
import { Link } from '@tribeplatform/react-ui-kit/Link'

import { MediaModal, Slide } from '../MediaModal'
import { ComposerModuleName } from './@types'
import { CodeCopyButton } from './CodeCopyButton'
import { renderAudio, renderImg, renderVideo } from './renderers'
import { imgToImage } from './utils'

const SyntaxHighlighter = lazy(() => import('react-syntax-highlighter'))

const parentIsAList = domNode =>
  ['li', 'ol', 'ul'].indexOf(domNode?.parent?.name) > -1

export const HtmlContent = ({
  value,
  embeds,
  mentions,
  isReply,
  generateMemberPath,
}: {
  value: string
  embeds?: Embed[]
  mentions?: Partial<Member>[]
  isReply?: boolean
  generateMemberPath?: (member: Partial<Member>) => string
}) => {
  let mediaIndex = -1
  // The first one is active by default.
  const [activeMediaIndex, setActiveMediaIndex] = useState(0)
  const slides: Slide[] = []
  const [mediaModalVisibility, setMediaModalVisibility] = useState(false)

  const handleModalVisibility = useCallback(e => {
    setActiveMediaIndex(Number(e?.currentTarget.dataset?.index))
    setMediaModalVisibility(true)
  }, [])
  const handleModalClose = useCallback(() => {
    setMediaModalVisibility(false)
  }, [])
  const embedAlignmentClass = isReply ? '' : '-ml-6 -mr-6'

  const options: HTMLReactParserOptions = {
    replace: _domNode => {
      const domNode = _domNode as any
      const tagName = domNode.name as string
      let codeLanguage = 'text'

      switch (tagName) {
        case 'figure': {
          switch (domNode.firstChild?.name) {
            case 'img': {
              const imgSlide = imgToImage(domNode.firstChild)
              if (imgSlide) {
                slides.push(imgSlide)
              }
              mediaIndex += 1
              return renderImg(domNode.firstChild, {
                className: parentIsAList(domNode) ? '' : embedAlignmentClass,
                mediaIndex,
                onClick: handleModalVisibility,
              })
            }
            case 'audio':
              return renderAudio(domNode.firstChild)
            case 'video':
              return renderVideo(domNode.firstChild, {
                className: embedAlignmentClass,
              })
            default:
              return domNode
          }
        }
        case 'img': {
          const imgSlide = imgToImage(domNode)
          if (imgSlide) {
            slides.push(imgSlide)
          }
          mediaIndex += 1
          return renderImg(domNode, {
            className: parentIsAList(domNode) ? '' : embedAlignmentClass,
            mediaIndex,
            onClick: handleModalVisibility,
          })
        }
        case 'h1':
        case 'h2':
          return (
            <h2 className="text-xl">
              {domToReact(domNode?.children, options)}
            </h2>
          )

        case 'audio':
          return renderAudio(domNode)
        case 'video':
          return renderVideo(domNode, { className: embedAlignmentClass })

        case 'pre': {
          if (
            !domNode?.children?.length ||
            domNode?.children[0]?.name !== 'code' ||
            domNode?.children[0]?.children?.length !== 1 ||
            domNode?.children[0]?.children[0]?.type !== 'text'
          )
            return null

          if (domNode?.children[0]?.attribs?.class) {
            codeLanguage = domNode?.children[0]?.attribs?.class.replace(
              'language-',
              '',
            )
          }

          const content = domToReact(domNode?.children[0].children)

          return (
            <div className={clsx('code-wrapper relative', embedAlignmentClass)}>
              <CodeCopyButton content={content as string} />
              {typeof window !== 'undefined' ? (
                <Suspense
                  fallback={
                    <pre>
                      <code>{content}</code>
                    </pre>
                  }
                >
                  <SyntaxHighlighter
                    language={codeLanguage}
                    style={atomOneDark}
                  >
                    {content}
                  </SyntaxHighlighter>
                </Suspense>
              ) : (
                <pre>
                  <code>{content}</code>
                </pre>
              )}
            </div>
          )
        }
        case 'h3':
          return (
            <h2 className="text-lg">
              {domToReact(domNode?.children, options)}
            </h2>
          )

        case 'ol':
          return (
            <ol
              start={domNode?.attribs?.start}
              className="list-decimal list-inside pl-6"
            >
              {domToReact(domNode?.children, options)}
            </ol>
          )

        case 'ul':
          return (
            <ul className="list-disc list-inside pl-6">
              {domToReact(domNode?.children, options)}
            </ul>
          )

        case 'div':
          if (
            domNode.type === 'tag' &&
            domNode.attribs?.['data-type'] === ComposerModuleName.Embed
          ) {
            if (!embeds) {
              return null
            }
            // Fetch the corresponding embed id
            const embedId = domNode.attribs['data-id']
            const embed = embeds.find(embed => embed.id === embedId)

            const embedRendered = (
              <>
                <div className="preview relative w-full">
                  <figure dangerouslySetInnerHTML={{ __html: embed.html }} />
                </div>
                <Helmet>
                  <script src="//cdn.iframe.ly/embed.js" async></script>
                </Helmet>
              </>
            )

            if (embed?.type === 'video')
              return (
                <div className={clsx(embedAlignmentClass)}>{embedRendered}</div>
              )

            return embedRendered
          }
          break

        case 'a':
          if (domNode.attribs?.['data-type'] === ComposerModuleName.Mention) {
            const mentionId = domNode.attribs['data-id']
            const mentionFromDb = mentions?.find?.(
              mention => mention.id === mentionId,
            )

            return (
              <Link as={RouterLink} to={generateMemberPath({ id: mentionId })}>
                {mentionFromDb?.name
                  ? `@${mentionFromDb.name}`
                  : domToReact(domNode?.children, options)}
              </Link>
            )
          }

          return (
            <Link
              target="_blank"
              rel="noopener noreferrer"
              href={domNode?.attribs?.href}
            >
              {domToReact(domNode?.children, options)}
            </Link>
          )

        default:
          // If null is returned parser will render the html variant.
          return null
      }

      return null
    },
  }

  const Component = Parser(value, options)
  return (
    <>
      {Component}
      {slides?.length > 0 && (
        <MediaModal
          slides={slides}
          isVisible={mediaModalVisibility}
          onClose={handleModalClose}
          activeMediaIndex={activeMediaIndex}
        />
      )}
    </>
  )
}
