import React, { useState, useEffect, useRef } from 'react'

import clsx from 'clsx'
import fileSize from 'filesize'
import { Carousel } from 'react-responsive-carousel'
import ArrowLeftLineIcon from 'remixicon-react/ArrowLeftLineIcon.js'
import ArrowRightLineIcon from 'remixicon-react/ArrowRightLineIcon.js'
import CloseLineIcon from 'remixicon-react/CloseLineIcon.js'
import DownloadCloud2LineIcon from 'remixicon-react/DownloadCloud2LineIcon.js'

import { File } from '@tribeplatform/gql-client/types'
import { Button } from '@tribeplatform/react-ui-kit/Button'
import { Modal } from '@tribeplatform/react-ui-kit/Modal'

import { MediaModalProps } from './@types'
import { MediaPreview } from './MediaPreview.js'
import { downloadFile } from './utils.js'

export const MediaModal = ({
  slides,
  isVisible = false,
  onClose,
  activeMediaIndex = 0,
}: MediaModalProps) => {
  const cache = useRef<{ [key: number]: boolean }>({})
  const [downloading, setDownloading] = useState(false)
  const [currentIndex, setCurrentIndex] = useState(activeMediaIndex)
  const slide = slides[currentIndex]
  const length = slides?.length
  const isPreviousButtonEnabled = currentIndex > 0
  const isNextButtonEnabled = currentIndex < length - 1

  const handleDownload = async () => {
    const slide = slides[currentIndex]
    if (!slide?.url && !slide?.downloadUrl) return
    setDownloading(true)

    if (slide.downloadUrl) {
      await downloadFile(slide.downloadUrl, slide.name)
    } else {
      await downloadFile(slide.url)
    }

    setDownloading(false)
  }

  useEffect(() => {
    return () => {
      // clear the cache on unmount
      cache.current = {}
    }
  }, [])

  useEffect(() => {
    setCurrentIndex(activeMediaIndex)
  }, [activeMediaIndex])

  return (
    <Modal
      size="full"
      open={isVisible}
      onClose={onClose}
      className="flex-col flex"
    >
      <div className="px-4 py-5 sm:px-6 flex items-center justify-end border-b border-surface-200/50 space-x-1">
        <div className="overflow-hidden flex-grow">
          {slide.name && <p className="truncate">{slide.name}</p>}
          {(slide as File).size && (
            <p className="text-basicSurface-500">
              {fileSize((slide as File).size)}
            </p>
          )}
        </div>
        <div className="flex">
          <Button
            size="md"
            className="hover:bg-transparent"
            variant="basic"
            disabled={downloading}
            loading={downloading}
            leadingIcon={<DownloadCloud2LineIcon />}
            onClick={handleDownload}
          />

          <Button
            size="md"
            className="hover:bg-transparent"
            variant="basic"
            onClick={onClose}
            leadingIcon={<CloseLineIcon />}
          />
        </div>
      </div>

      <Modal.Content
        className={clsx(
          'flex items-center justify-center space-x-3 py-0',
          length > 1 ? 'px-3' : 'px-0 sm:p-0',
        )}
      >
        {length > 1 && (
          <Button
            variant="outline"
            rounded
            onClick={() =>
              isPreviousButtonEnabled && setCurrentIndex(currentIndex - 1)
            }
            disabled={!isPreviousButtonEnabled}
            leadingIcon={<ArrowLeftLineIcon />}
          />
        )}

        <div className="max-h-screen flex-1">
          <Carousel
            selectedItem={currentIndex}
            dynamicHeight={false}
            showStatus={false}
            infiniteLoop={false} // This should remain false otherwise the lazy-load won't work
            showThumbs={false}
            showIndicators={false}
            showArrows={false}
            transitionTime={200}
            useKeyboardArrows
          >
            {slides.map((slide, index) => {
              if (cache.current[index] || index === currentIndex) {
                cache.current[currentIndex] = true
                // Other types like video will be added later.
                return (
                  <MediaPreview
                    key={slide.id || slide.name}
                    media={slide}
                    isActive={currentIndex === index}
                  />
                )
              }
              return (
                <div
                  className="flex items-center justify-center"
                  key={slide.id || slide.name}
                />
              )
            })}
          </Carousel>
        </div>

        {length > 1 && (
          <Button
            variant="outline"
            rounded
            onClick={() =>
              isNextButtonEnabled && setCurrentIndex(currentIndex + 1)
            }
            disabled={!isNextButtonEnabled}
            leadingIcon={<ArrowRightLineIcon />}
          />
        )}
      </Modal.Content>
    </Modal>
  )
}
