import { QueryClient } from 'react-query'

import {
  Collection,
  Page,
  QuerySpaceArgs,
  Space,
} from '@tribeplatform/gql-client/types'

import { useQueryClient } from '../../lib'
import {
  getAuthTokenKey,
  getCollectionsKey,
  getPageKey,
  getSpaceKey,
  getSpacesByIdsKey,
  getSpacesBySlugsKey,
  getSpacesKey,
} from '../../utils/keys'

const findSpaceInSystemSpaces = (args: QuerySpaceArgs, queryClient): Space => {
  let cachedSpace = null

  const queries = queryClient.getQueriesData({
    queryKey: getAuthTokenKey(),
  })
  queries?.forEach(query => {
    const queryData = query[1] as {
      network: { systemSpaces?: [{ id; slug; address }] }
    }
    queryData?.network?.systemSpaces?.forEach(space => {
      if (
        space?.id === args.id ||
        space?.slug === args.slug ||
        space?.address?.path === args.path
      ) {
        cachedSpace = { ...space, ...cachedSpace }
      }
    })
  })

  return cachedSpace && Object.keys(cachedSpace).length > 0
    ? cachedSpace
    : undefined
}

const findSpace = (args: QuerySpaceArgs, queryClient): Space | undefined => {
  let cachedSpace = null

  const queries = queryClient.getQueriesData({
    queryKey: getSpaceKey(),
    active: true,
  })
  queries?.forEach(query => {
    const space = query[1] as { id; slug; address }
    if (
      space?.id === args.id ||
      space?.slug === args.slug ||
      space?.address?.path === args.path
    ) {
      cachedSpace = { ...space, ...cachedSpace }
    }
  })

  return cachedSpace && Object.keys(cachedSpace).length > 0
    ? cachedSpace
    : undefined
}

const findSpaceInIds = (args: QuerySpaceArgs, queryClient): Space => {
  let cachedSpace = null

  const queries = queryClient.getQueriesData({
    queryKey: getSpacesByIdsKey(),
    active: true,
  })
  queries?.forEach(query => {
    const queryData = query[1] as [{ id; slug; address }]
    queryData?.forEach(space => {
      if (
        space?.id === args.id ||
        space?.slug === args.slug ||
        space?.address?.path === args.path
      ) {
        cachedSpace = { ...space, ...cachedSpace }
      }
    })
  })

  return cachedSpace && Object.keys(cachedSpace).length > 0
    ? cachedSpace
    : undefined
}

const findSpaceInSlugs = (args: QuerySpaceArgs, queryClient): Space => {
  let cachedSpace = null

  const queries = queryClient.getQueriesData({
    queryKey: getSpacesBySlugsKey(),
    active: true,
  })
  queries?.forEach(query => {
    const queryData = query[1] as [{ id; slug; address }]
    queryData?.forEach(space => {
      if (
        space?.id === args.id ||
        space?.slug === args.slug ||
        space?.address?.path === args.path
      ) {
        cachedSpace = { ...space, ...cachedSpace }
      }
    })
  })

  return cachedSpace && Object.keys(cachedSpace).length > 0
    ? cachedSpace
    : undefined
}

const findSpaceInSpaces = (args: QuerySpaceArgs, queryClient): Space => {
  let cachedSpace = null

  const queries = queryClient.getQueriesData({
    queryKey: getSpacesKey(),
    active: true,
  })
  queries?.forEach(query => {
    const queryData = query[1] as {
      pages: [{ nodes: [{ id; slug; address }] }]
    }
    queryData?.pages?.forEach(page => {
      page?.nodes?.forEach(space => {
        if (
          space?.id === args.id ||
          space?.slug === args.slug ||
          space?.address?.path === args.path
        ) {
          cachedSpace = { ...space, ...cachedSpace }
        }
      })
    })
  })

  return cachedSpace && Object.keys(cachedSpace).length > 0
    ? cachedSpace
    : undefined
}

const findSpaceInCollections = (args: QuerySpaceArgs, queryClient): Space => {
  let cachedSpace = null

  const queries = queryClient.getQueriesData({
    queryKey: getCollectionsKey(),
    active: true,
  })
  queries?.forEach(query => {
    const queryData = query[1] as Collection[]

    queryData?.forEach(collection => {
      collection?.spaces?.nodes?.forEach(space => {
        if (
          space?.id === args.id ||
          space?.slug === args.slug ||
          space?.address?.path === args.path
        ) {
          cachedSpace = { ...space, ...cachedSpace }
        }
      })
    })
  })

  return cachedSpace && Object.keys(cachedSpace).length > 0
    ? cachedSpace
    : undefined
}

const findSpaceInPages = (args: QuerySpaceArgs, queryClient): Space => {
  let cachedSpace = null

  const queries = queryClient.getQueriesData({
    queryKey: getPageKey(),
    active: true,
  })
  queries?.forEach(query => {
    const page = query[1] as Page
    if (page?.space) {
      const { space } = page
      if (
        space?.id === args.id ||
        space?.slug === args.slug ||
        space?.address?.path === args.path
      ) {
        cachedSpace = { ...space, ...cachedSpace }
      }
    }
  })

  return cachedSpace && Object.keys(cachedSpace).length > 0
    ? cachedSpace
    : undefined
}

export const getCachedSpace = (
  args: QuerySpaceArgs,
  queryClient: QueryClient,
): Space => {
  const space =
    findSpaceInSystemSpaces(args, queryClient) ||
    findSpace(args, queryClient) ||
    findSpaceInIds(args, queryClient) ||
    findSpaceInSlugs(args, queryClient) ||
    findSpaceInSpaces(args, queryClient) ||
    findSpaceInCollections(args, queryClient) ||
    findSpaceInPages(args, queryClient)

  return space && Object.keys(space).length ? space : null
}

export const useCachedSpace = (args: QuerySpaceArgs): Space => {
  const queryClient = useQueryClient()
  return getCachedSpace(args, queryClient)
}

export const useCachedSpaces = (args: QuerySpaceArgs[]): Space[] => {
  const queryClient = useQueryClient()
  return args
    .map(arg => getCachedSpace(arg, queryClient))
    .filter(space => space)
}
