import { NewThemeFields } from '@tribeplatform/gql-client'
import {
  AuthToken,
  MutationUpsertThemeArgs,
  Network,
  NewTheme,
} from '@tribeplatform/gql-client/types'

import { useMutation, UseMutationOptions, useQueryClient } from '../../lib'
import { useTribeClient } from '../../useTribeClient'
import { getAuthTokenKey, getNetworkKey } from '../../utils/keys'
import { authTokenUpdater, networkUpdater, useCachedNetwork } from '../cache'

export const useUpsertTheme = (options?: {
  fields?: NewThemeFields
  useMutationOptions?: UseMutationOptions<
    NewTheme,
    Error,
    MutationUpsertThemeArgs,
    { snapshot: Network }
  >
}) => {
  const { useMutationOptions, fields } = options || {}
  const { client } = useTribeClient()
  const queryClient = useQueryClient()
  const networkKey = getNetworkKey()
  const authTokenKey = getAuthTokenKey()
  const networkSnapshot = useCachedNetwork()

  return useMutation<
    NewTheme,
    Error,
    MutationUpsertThemeArgs,
    { snapshot: Network }
  >((input: MutationUpsertThemeArgs) => client.theme.upsert(input, fields), {
    onMutate: async newTheme => {
      const snapshot = networkSnapshot

      if (newTheme.input?.active) {
        await queryClient.cancelQueries(networkKey)

        queryClient.setQueriesData<Network>(
          networkKey,
          networkUpdater({ activeTheme: newTheme.input }),
        )
        queryClient.setQueriesData<AuthToken>(
          authTokenKey,
          authTokenUpdater({
            network: { activeTheme: newTheme.input },
          }),
        )
      }

      return { snapshot }
    },
    onSuccess: newTheme => {
      queryClient.setQueriesData<AuthToken>(
        authTokenKey,
        authTokenUpdater({
          network: { activeTheme: newTheme },
        }),
      )
    },
    onError: (err, newTheme, context) => {
      queryClient.setQueriesData<Network>(networkKey, context.snapshot)
      queryClient.setQueriesData<AuthToken>(
        authTokenKey,
        authTokenUpdater({ network: context.snapshot }),
      )
    },
    onSettled: () => {
      queryClient.invalidateQueries(networkKey)
      queryClient.invalidateQueries(authTokenKey)
    },
    ...useMutationOptions,
  })
}
